#include "Timespec.h"


#define ONE_SECOND_IN_NANOSECONDS 1000000000L

namespace DivaCore
{
  Timespec::Timespec (const struct timespec & t)
    : m_ts(t)
  {
    normalize();
  }

  Timespec::Timespec ()
  {
    m_ts.tv_sec = 0;
    m_ts.tv_nsec = 0;
  }

  void Timespec::normalize()
  {
    while (m_ts.tv_nsec >= ONE_SECOND_IN_NANOSECONDS)
      {
	m_ts.tv_nsec -= ONE_SECOND_IN_NANOSECONDS;
	m_ts.tv_sec++;
      }      
  }

  void Timespec::addNanoSeconds(const long delta_ns)
  {
    if (delta_ns > 0)
      {
	m_ts.tv_nsec += delta_ns;
	normalize();
      }
  }

  void Timespec::addSeconds(const long delta_s)
  {
    if (delta_s > 0)
      {
	m_ts.tv_sec += delta_s;
	normalize();
      }
  }

  void Timespec::add(const Timespec & ts)
  {
    if (!ts.isNull())
      {
	m_ts.tv_sec  += ts.seconds();
	m_ts.tv_nsec += ts.nanoSeconds();
	normalize();
      }
  }

  void Timespec::subtract(const Timespec & ts)
  {
    if (!ts.isNull())
      {
	//-- we do not allow negative seconds.
	if (ts.seconds() > m_ts.tv_sec)
	{
	  return;
	}

	m_ts.tv_sec  -= ts.seconds();
	m_ts.tv_nsec -= ts.nanoSeconds();
	while (m_ts.tv_nsec < 0)
	{
	    --m_ts.tv_sec;
	    m_ts.tv_nsec += ONE_SECOND_IN_NANOSECONDS;
	}
      }
  }

  bool Timespec::operator > (const Timespec & other)
  {
    return (seconds() > other.seconds())
      || (seconds() == other.seconds() && nanoSeconds() > other.nanoSeconds());
  }

  bool Timespec::operator < (const Timespec & other)
  {
    return (seconds() < other.seconds())
      || (seconds() == other.seconds() && nanoSeconds() < other.nanoSeconds());
  }

  bool Timespec::operator == (const Timespec & other)
  {
    return seconds()==other.seconds() && nanoSeconds() == other.nanoSeconds();
  }

  bool Timespec::isNull() const
  {
    return (m_ts.tv_sec==0) && (m_ts.tv_nsec==0);
  }

  bool Timespec::setToNow(const int id)
  {
    return (clock_gettime(id, ptr()) == 0);
  }

  bool Timespec::sleepUntil(Timespec & remain) const
  {
    return clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, const_ptr(), remain.ptr()) == 0;
  }

};
