#ifndef __DIVA_TIMESPEC_H__
#define __DIVA_TIMESPEC_H__

#include <stdlib.h>
#include <time.h>
#include <sched.h>
#include <sys/types.h>
#include <unistd.h>

namespace DivaCore
{
  class Timespec
  {
  public:
    Timespec (const struct timespec & t);
    Timespec ();

    /// ensure that nanoseconds is less than one second.
    void normalize();

    /// Add some nanoseconds.
    void addNanoSeconds(const long delta_ns);

    void addSeconds(const long delta_s);

    // Add another Timespec
    void add(const Timespec & ts);

    void subtract(const Timespec & ts);

    bool operator > (const Timespec & other);

    bool operator < (const Timespec & other);

    bool operator == (const Timespec & other);


    bool isNull() const;

    unsigned long seconds() const { return m_ts.tv_sec; }
    unsigned long nanoSeconds() const { return m_ts.tv_nsec; }

    const struct timespec & timespec () const { return m_ts; }
    struct timespec * ptr () { return &m_ts; }
    const struct timespec * const_ptr () const { return &m_ts; }


  public: // OS specific methods
    bool setToNow(const int id = CLOCK_MONOTONIC);
    
    bool sleepUntil(Timespec & remain) const;

  private:
    struct timespec m_ts;
  };
};

#endif
